/*
 *  Cryptographic "Data-as-Key" Feedback Analyser
 *
 *  Copyright (C) 2005, 2008 Luke Kenneth Casson Leighton
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#define K_SZ 16
#define D_SZ 16

#ifdef TEST_DES
#undef K_SZ
#undef D_SZ
#define K_SZ 7
#define D_SZ 8

#endif

#include <string.h>
#include <unistd.h>
#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <getopt.h>

#include "util.h"

struct bitpair
{
	int k1;
	int d1;
	int k2;
	int d2;
};

struct bitpairs
{
	struct bitpair *pairs;
	int num_pairs;
};

struct keyblock_pair
{
	unsigned char key[K_SZ];
	unsigned char block[D_SZ];
};

struct keyblock_pairs
{
	struct keyblock_pair *pairs;
	int num_pairs;
};

/*
 * this is an implementation in c of process_ratios in interact.py
 */
void process_ratios(  const struct bitpairs *bits,
		const struct keyblock_pairs *keyblocks,
		int key_prob[K_SZ]
		)
{
	int kc;
	int i;

	for (kc = 0; kc < keyblocks->num_pairs; kc++)
	{
		const struct keyblock_pair *kbp = &(keyblocks->pairs[kc]);
		for (i = 0; i < bits->num_pairs; i++)
		{
			const struct bitpair *bp = &(bits->pairs[i]);
			int set = bit_is_set((unsigned char*)kbp->key, bp->k1) ^
			          bit_is_set((unsigned char*)kbp->key, bp->k2) ^
			          bit_is_set((unsigned char*)kbp->block, bp->d1) ^
			          bit_is_set((unsigned char*)kbp->block, bp->d2);
			if (!set)
			{
				key_prob[bp->k1] += 1.0;
				key_prob[bp->k2] += 1.0;
			}
			else
			{
				key_prob[bp->k1] -= 1.0;
				key_prob[bp->k2] -= 1.0;
			}
		}
	}
}

int main( int ac, char *av[] )
{
	char c;
	int k;
	struct keyblock_pairs keyblocks;
	FILE *vector_file = stdin;

	keyblocks.num_pairs = 2560;

    while( (c = getopt( ac, av, "v:l:")) != EOF )
	{
		switch(c)
		{
			case 'l':
			{
				keyblocks.num_pairs = atoi(optarg);
				break;
			}
			case 'v':
			{
				vector_file = fopen(optarg, "r");
				break;
			}
		}
	}

#if 0
	if (interactive && !read_key)
	{
		for (i = 0; i < K_SZ*8; i++)
		{
			fscanf(stdin, "%f\n", &keyprob[i]);
			printf("%d %f\n", i, keyprob[i]);
			fflush(stdout);
		}
	}
#endif

	keyblocks.pairs = malloc(sizeof(struct keyblock_pair) * keyblocks.num_pairs);
	printf("num keyblocks: %d\n", keyblocks.num_pairs);
	fflush(stdout);

	for (k = 0; k < keyblocks.num_pairs; k++) /* the more, the merrier */
	{
		struct keyblock_pair *kbp = &(keyblocks.pairs[k]);
		read_input(kbp->key, K_SZ);
		read_input(kbp->block, D_SZ);
		printf("read input\n");
		fflush(stdout);
	}

	printf("read pairs\n");
	fflush(stdout);

	while (!feof(stdin))
	{
		struct bitpairs bits;
		int key_prob[K_SZ*8];
		int bp;

		for (k = 0; k < K_SZ*8; k++)
			key_prob[k] = 0;

		fscanf(stdin, "%d", &bits.num_pairs);
		printf("num pairs: %d\n", bits.num_pairs);
		fflush(stdout);

		if (bits.num_pairs == 0)
			break;

		bits.pairs = malloc(sizeof(struct bitpair) * bits.num_pairs);

		for (bp = 0; bp < bits.num_pairs; bp++)
		{
			struct bitpair *kbp = &(bits.pairs[bp]);
			fscanf(vector_file, "%d", &kbp->k1);
			fscanf(vector_file, "%d", &kbp->d1);
			fscanf(vector_file, "%d", &kbp->k2);
			fscanf(vector_file, "%d", &kbp->d2);
			printf("read bitpairs\n");
			#if 0
			printf("read bitpairs\n%d %d %d %d\n",
				kbp->k1, kbp->d1, kbp->k2, kbp->d2);
				#endif
			fflush(stdout);
		}
		process_ratios(  &bits, &keyblocks, key_prob);
		for (k = 0; k < K_SZ*8; k++)
			printf("%d\n", key_prob[k]);
		fflush(stdout);

	}
#if 0
		memset( input, 0x0, D_SZ );
		/*bit_set(input, k);*/
		(*(int*)input) = k;
#endif

/*
		if (output_baseline)
		{
			write_data(baseline, D_SZ);
			printf("encrypt\n");
			fflush(stdout);
		}
			*/
/*		write_floatbucket(pval_bitbucket, fn, k);*/

	return( 0 );
}

